#-----------------------------------------------------------------------------
# Smoother
# Copyright (c) 2006 David G. Drumright (ockham)
# Jan 26, 2006

# Highlight desired body parts in the list; or to apply to all parts,
# just leave the list alone.

# When you hit Go, the script looks for 'outlier' keyframes and eliminates them.

# Note that the listbox has special entries for "full right hand" and "full left hand".
# If you want to smooth either hand with its fingers, use these.  If you want
# to pick out specific finger joints, you'll still find them further down in the list.
#-----------------------------------------------------------------------------

from Tkinter import *
import poser
scene = poser.Scene()

TheFig=scene.CurrentFigure()
if not TheFig: raise "No figure selected"

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Smoother")

        self.StatusEntry = Entry(self.master,width=50)
        self.StatusEntry.grid(row=0,column=0,columnspan=2)
        self.StatusEntry.insert(0,"Pick parts, hit Go")

        # Lists
        self.ListScroll = Scrollbar(self.master, orient=VERTICAL)
        self.ListScroll.grid( row=2, column=0,sticky=N+S+E)
        self.List = Listbox(self.master, height=15, width=25,selectmode=MULTIPLE,yscrollcommand = self.ListScroll.set,exportselection=0)
        self.List.grid( row=2, column=1)
        # Set the scrollbar to work
        self.ListScroll["command"] = self.List.yview

        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text="Go", command=self.HandleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=3, column=1)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

        # Fill the list with body parts of current figure, with special Hand entries.
        self.List.insert(END,"Full Hand Left")
        self.List.insert(END,"Full Hand Right")
        for OneActor in TheFig.Actors():
            self.List.insert(END,OneActor.Name())

    # - - - - - - - - - - - - - - - - - -
    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        if scene: scene.ProcessSomeEvents(1)
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -

    def HandleGo(self):
        Sels=self.List.curselection()
        if not Sels:
            # Do all the items except the special FullLeft and FullRight, which are the first two.
            for i in range(2,self.List.size()):
                PartName = self.List.get(i)
                self.Smooth(PartName)
        else:
            for S in Sels:
                PartName = self.List.get(int(S))
                if   PartName=="Full Hand Left":  self.SmoothFullHandLeft()
                elif PartName=="Full Hand Right": self.SmoothFullHandRight()
                else:                             self.Smooth(PartName)

        scene.SetFrame(0)
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleMore(self):
        Sels=self.List.curselection()
        if not Sels:
            # Do all the items except the special FullLeft and FullRight, which are the first two.
            for i in range(2,self.List.size()):
                PartName = self.List.get(i)
                self.Smooth(PartName)
        else:
            for S in Sels:
                PartName = self.List.get(int(S))
                if   PartName=="Full Hand Left":  self.SmoothFullHandLeft()
                elif PartName=="Full Hand Right": self.SmoothFullHandRight()
                else:                             self.Smooth(PartName)

        scene.SetFrame(0)
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def Smooth(self,ActorName):
        # Go through all keyframes.  Cut out frames that are
        # 'outliers'.  Specifically, if the value at the frame is
        # not between the values of its neighboring frames, eliminate it.

        # To avoid possible interference between the changes and the actual,
        # build an array of the frames to delete first; then go through and delete them.
        TheActor=TheFig.Actor(ActorName)
        for OneParm in TheActor.Parameters():
            if OneParm.Hidden(): continue
            self.ShowStatus("working on actor %s parm %s" % (ActorName,OneParm.Name()) )
            Last=scene.NumFrames()-1
            ActualFrames=[]
            NeedsDelete=[]

            # Begin by getting a list of the actual keyframes so we can work
            # on that basis instead of raw frames.  This will make it possible
            # to use the script repeatedly.
            F=1
            scene.SetFrame(F)
            tryF=OneParm.NextKeyFrame() # See if there's anything after the first.
            if not tryF: continue # If this parm has no keyframes after
            # the first, jump to next parm.

            # Implied else:
            F=1
            scene.SetFrame(F)
            while F < Last:
                tryF=OneParm.NextKeyFrame()
                if not tryF:  break  # No next.
                else:         F=tryF
                scene.SetFrame(F)
                ActualFrames.append(F)
            if len(ActualFrames) < 1: continue

            # Now do the real run through these frames.
            for F in ActualFrames:
                scene.SetFrame(F)
                ThisVal=OneParm.Value()

                # Look at its neighbors.  Check to be sure
                # we're not beyond ends.
                Prev=OneParm.PrevKeyFrame()
                if not Prev:
                    continue
                Next=OneParm.NextKeyFrame()
                if not Next: # If no further here, we're at end.
                    break
                if Next >= Last: break

                # After testing, get the values:
                scene.SetFrame(Prev)
                PrevVal=OneParm.Value()
                scene.SetFrame(Next)
                NextVal=OneParm.Value()

                if   (ThisVal > PrevVal) and (ThisVal > NextVal): NeedsDelete.append(F)
                elif (ThisVal < PrevVal) and (ThisVal < NextVal): NeedsDelete.append(F)
            # end loop across frames in one parm.

            # Now go through and use the list.
            for F in NeedsDelete:
                scene.SetFrame(F)
                OneParm.DeleteKeyFrame(F)
        # End loop across parameters

    # - - - - - - - - - - - - - - - - - -

    def SmoothFullHandLeft(self):
        # Just use all the standard names for left hand, and run the smooth for all
        LParts = ["lHand",
                  "lThumb1",  "lThumb2",  "lThumb3",
                  "lIndex1",  "lIndex2",  "lIndex3",
                  "lMid1",    "lMid2",    "lMid3",
                  "lRing1",   "lRing2",   "lRing3",
                  "lPinky1",  "lPinky2",  "lPinky3"]
        for OneActor in LParts:
            self.Smooth(OneActor)

    # - - - - - - - - - - - - - - - - - -

    def SmoothFullHandRight(self):
        # Just use all the standard names for right hand, and run the smooth for all
        RParts = ["rHand",
                  "rThumb1", "rThumb2", "rThumb3",
                  "rIndex1", "rIndex2", "rIndex3",
                  "rMid1",   "rMid2",   "rMid3",
                  "rRing1",  "rRing2",  "rRing3",
                  "rPinky1", "rPinky2", "rPinky3"]
        for OneActor in RParts:
            self.Smooth(OneActor)

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
