#-----------------------------------------------------------------------------
# Smoother
# Copyright (c) 2006 David G. Drumright (ockham)
# Jan 26, 2006
# Simplified version for Mac.

#-----------------------------------------------------------------------------

import poser
scene = poser.Scene()

TheFig=scene.CurrentFigure()
if not TheFig: raise "No figure selected"

#-----------------------------------------------------------------------------
#------------------------------------------------------------

def Smooth(ActorName):
    # Go through all keyframes.  Cut out frames that are
    # 'outliers'.  Specifically, if the value at the frame is
    # not between the values of its neighboring frames, eliminate it.

    # To avoid possible interference between the changes and the actual,
    # build an array of the frames to delete first; then go through and delete them.
    TheActor=TheFig.Actor(ActorName)
    for OneParm in TheActor.Parameters():
        if OneParm.Hidden(): continue
        Last=scene.NumFrames()-1
        ActualFrames=[]
        NeedsDelete=[]

        # Begin by getting a list of the actual keyframes so we can work
        # on that basis instead of raw frames.  This will make it possible
        # to use the script repeatedly.
        F=1
        scene.SetFrame(F)
        tryF=OneParm.NextKeyFrame() # See if there's anything after the first.
        if not tryF: continue # If this parm has no keyframes after
        # the first, jump to next parm.

        # Implied else:
        F=1
        scene.SetFrame(F)
        while F < Last:
            tryF=OneParm.NextKeyFrame()
            if not tryF:  break  # No next.
            else:         F=tryF
            scene.SetFrame(F)
            ActualFrames.append(F)
        if len(ActualFrames) < 1: continue

        # Now do the real run through these frames.
        for F in ActualFrames:
            scene.SetFrame(F)
            ThisVal=OneParm.Value()

            # Look at its neighbors.  Check to be sure
            # we're not beyond ends.
            Prev=OneParm.PrevKeyFrame()
            if not Prev:
                continue
            Next=OneParm.NextKeyFrame()
            if not Next: # If no further here, we're at end.
                break
            if Next >= Last: break

            # After testing, get the values:
            scene.SetFrame(Prev)
            PrevVal=OneParm.Value()
            scene.SetFrame(Next)
            NextVal=OneParm.Value()

            if   (ThisVal > PrevVal) and (ThisVal > NextVal): NeedsDelete.append(F)
            elif (ThisVal < PrevVal) and (ThisVal < NextVal): NeedsDelete.append(F)
        # end loop across frames in one parm.

        # Now go through and use the list.
        for F in NeedsDelete:
            scene.SetFrame(F)
            OneParm.DeleteKeyFrame(F)
    # End loop across parameters

#------------------------------------------------------------

for OneActor in TheFig.Actors():
    Name=OneActor.Name()
    UName=string.upper(Name)
    if string.count(UName,'GOAL') or string.count(UName,'CENTER'): continue
    print "working on actor %s" % Name
    Smooth(Name)

#------------------------------------------------------------
